/*
   Copyright The containerd Authors.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package types

import "fmt"

// CollapsibleError indicates the error should be collapsed
type CollapsibleError interface {
	CollapseError()
}

// CollapsedError returns a new error with the collapsed
// error returned on unwrapped or when formatted with "%+v"
func CollapsedError(err error, collapsed ...error) error {
	return collapsedError{err, collapsed}
}

type collapsedError struct {
	error
	collapsed []error
}

func (c collapsedError) Unwrap() []error {
	return append([]error{c.error}, c.collapsed...)
}

func (c collapsedError) Format(s fmt.State, verb rune) {
	switch verb {
	case 'v':
		if s.Flag('+') {
			fmt.Fprintf(s, "%+v", c.error)
			for _, err := range c.collapsed {
				fmt.Fprintf(s, "\n%+v", err)
			}
			return
		}
		fallthrough
	case 's':
		fmt.Fprint(s, c.Error())
	case 'q':
		fmt.Fprintf(s, "%q", c.Error())
	}
}
